@@ -11,8 +11,7 @@
 #include "jammo-track-view.h"
 #include "jammo-sample-button.h"
 #include "../meam/jammo-sample.h"
-#include "jammo-mentor.h"
-
+#include "../gems/gems.h"
 G_DEFINE_TYPE(JammoTrackView, jammo_track_view, TANGLE_TYPE_DROPPABLE_ACTOR);
 
 enum {
@@ -20,36 +19,44 @@ enum {
 	PROP_TRACK,
 	PROP_N_SLOTS,
 	PROP_SLOT_DURATION,
-	PROP_SLOT_SIZE,
+	PROP_SLOT_HEIGHT,
+	PROP_SLOT_WIDTH,
 	PROP_LINE_EVERY_NTH_SLOT,
 	PROP_DISABLED_SLOTS_BEGIN,
-	PROP_DISABLED_SLOTS_END
+	PROP_DISABLED_SLOTS_END,
+	PROP_EDITING_ENABLED
 };
 
 struct _JammoTrackViewPrivate {
 	JammoEditingTrack* track;
 	guint n_slots;
 	guint64 slot_duration;
-	gfloat slot_size;
+	gfloat slot_width;
+	gfloat slot_height;
 	guint line_every_nth_slot;
 	guint disabled_slots_begin;
 	guint disabled_slots_end;
 	ClutterColor hilight_color;
 	
-	gfloat real_slot_size;
+	gfloat real_slot_width;
 	gfloat hilighted_start;
 	gfloat hilighted_end;
+	TangleGrid* grid;
+	
+	guint editing_enabled : 1;
 };
 
 static void on_actor_added(TangleWidget* widget, ClutterActor* actor, gpointer user_data);
 static void on_actor_removed(TangleWidget* widget, ClutterActor* actor, gpointer user_data);
-static void move_others_right_if_overlapping(JammoTrackView* track_view, ClutterActor* sample_button, guint slot_index);
+static gboolean check_if_overlapping(JammoTrackView* track_view, ClutterActor* sample_button, guint slot_index);
 
-ClutterActor* jammo_track_view_new(JammoEditingTrack* track, guint n_slots, guint64 slot_duration, gfloat slot_size) {
+ClutterActor* jammo_track_view_new(JammoEditingTrack* track, guint n_slots, guint64 slot_duration, gfloat slot_width, gfloat slot_height) {
 
-	return CLUTTER_ACTOR(g_object_new(JAMMO_TYPE_TRACK_VIEW, "track", track, "n-slots", n_slots, "slot-duration", slot_duration, "slot-size", slot_size, NULL));
+	//printf ("JAMMO_TYPE_TRACK_VIEW, n-slots:%d, slot-duration:%Lu,  slot-width:%f, slot-height:%f\n", n_slots,slot_duration, slot_width, slot_height);
+	return CLUTTER_ACTOR(g_object_new(JAMMO_TYPE_TRACK_VIEW, "track", track, "n-slots", n_slots, "slot-duration", slot_duration, "slot-width", slot_width, "slot-height", slot_height, NULL));
 }
 
+/* Note! This function does not check if the given sample will be overlapping some other sample. */
 void jammo_track_view_add_jammo_sample_button(JammoTrackView* track_view, JammoSampleButton* sample_button, guint slot) {
 	TangleWidget* widget;
 
@@ -57,17 +64,64 @@ void jammo_track_view_add_jammo_sample_button(JammoTrackView* track_view, JammoS
 
 	widget = TANGLE_WIDGET(tangle_wrapper_actor_get_wrapped(TANGLE_WRAPPER_ACTOR(track_view)));
 
-	clutter_actor_set_x(CLUTTER_ACTOR(sample_button), slot * track_view->priv->real_slot_size);
+	clutter_actor_set_x(CLUTTER_ACTOR(sample_button),
+		slot * (track_view->priv->real_slot_width ? track_view->priv->real_slot_width : track_view->priv->slot_width));
 	clutter_actor_set_y(CLUTTER_ACTOR(sample_button), 0.0);
+	clutter_actor_set_height(CLUTTER_ACTOR(sample_button), track_view->priv->slot_height);
 	tangle_widget_add(TANGLE_WIDGET(widget), CLUTTER_ACTOR(sample_button), NULL);
+}
 
-	move_others_right_if_overlapping(track_view, CLUTTER_ACTOR(sample_button), slot);
+/*
+Used when removing over network,
+*/
+void jammo_track_view_remove_jammo_sample_button_from_slot(JammoTrackView* track_view, guint asked_slot) {
+	gfloat asked_x = asked_slot * track_view->priv->slot_width;
+	//printf("looking sample-button from x-position '%f'\n",asked_x);
+	TangleWidget* widget;
+	GList* children;
+	ClutterActor* actor = NULL;
+
+	gfloat x;  //left side of actor
+	gfloat x2; //x2=x+width, right side of actor
+	widget = TANGLE_WIDGET(tangle_wrapper_actor_get_wrapped(TANGLE_WRAPPER_ACTOR(track_view)));
+
+	for (children = tangle_widget_get_children_readonly(widget); children; children = children->next) {
+		actor = CLUTTER_ACTOR(children->data);
+		x = clutter_actor_get_x(actor);
+		x2 = x + clutter_actor_get_width(actor);
+		//printf("this sample-button covers '%f' ... '%f'\n",x,x2);
+		if (x<= asked_x && asked_x < x2) {
+			//printf("found\n");
+			clutter_container_remove_actor(CLUTTER_CONTAINER(clutter_actor_get_parent(actor)), actor);
+			break;
+		}
+	}
 }
 
 guint jammo_track_view_get_sample_button_slot(JammoTrackView* track_view, JammoSampleButton* sample_button) {
 	g_return_val_if_fail(clutter_actor_get_parent(CLUTTER_ACTOR(sample_button)) == tangle_wrapper_actor_get_wrapped(TANGLE_WRAPPER_ACTOR(track_view)), (guint)-1);
 
-	return (guint)(clutter_actor_get_x(CLUTTER_ACTOR(sample_button)) / track_view->priv->real_slot_size);
+	return (guint)(clutter_actor_get_x(CLUTTER_ACTOR(sample_button)) / track_view->priv->real_slot_width);
+}
+
+gboolean jammo_track_view_get_editing_enabled(JammoTrackView* track_view) {
+
+	return track_view->priv->editing_enabled;
+}
+
+void jammo_track_view_set_editing_enabled(JammoTrackView* track_view, gboolean editing_enabled) {
+	GList* children;
+
+	if (track_view->priv->editing_enabled != editing_enabled) {
+		track_view->priv->editing_enabled = editing_enabled;
+		
+		tangle_droppable_actor_set_dropping_enabled(TANGLE_DROPPABLE_ACTOR(track_view), editing_enabled);
+		for (children = tangle_widget_get_children_readonly(TANGLE_WIDGET(tangle_wrapper_actor_get_wrapped(TANGLE_WRAPPER_ACTOR(track_view)))); children; children = children->next) {
+			tangle_draggable_actor_set_dragging_enabled(TANGLE_DRAGGABLE_ACTOR(children->data), editing_enabled);
+		}
+		
+		g_object_notify(G_OBJECT(track_view), "editing-enabled");
+	}
 }
 
 static gboolean jammo_track_view_is_dragging_droppable(TangleDroppableActor* droppable_actor, TangleDragging* dragging) {
@@ -82,17 +136,18 @@ static gboolean jammo_track_view_is_dragging_droppable(TangleDroppableActor* dro
 	
 	if (JAMMO_IS_SAMPLE_BUTTON(dragging->draggable_actor) &&
 	    clutter_actor_transform_stage_point(CLUTTER_ACTOR(track_view), dragging->center_x, dragging->center_y, &center_x, &center_y)) {
-		n_slots = (guint)(clutter_actor_get_width(CLUTTER_ACTOR(dragging->draggable_actor)) / track_view->priv->real_slot_size + 0.5);
-		center_x -= (n_slots - 1) * track_view->priv->real_slot_size / 2;
-		slot = center_x / track_view->priv->real_slot_size;
+		n_slots = (guint)(clutter_actor_get_width(CLUTTER_ACTOR(dragging->draggable_actor)) / track_view->priv->real_slot_width + 0.5);
+		center_x -= (n_slots - 1) * track_view->priv->real_slot_width / 2;
+		slot = center_x / track_view->priv->real_slot_width;
 
 		if (slot >= track_view->priv->disabled_slots_begin &&
-		    slot + n_slots - 1 < track_view->priv->n_slots - track_view->priv->disabled_slots_end) {
+		    slot + n_slots - 1 < track_view->priv->n_slots - track_view->priv->disabled_slots_end &&
+		    !check_if_overlapping(track_view, CLUTTER_ACTOR(dragging->draggable_actor), slot)) {
 			droppable = TRUE;
-			start = slot * track_view->priv->real_slot_size;
+			start = slot * track_view->priv->real_slot_width;
 			if (start != track_view->priv->hilighted_start) {
 				track_view->priv->hilighted_start = start;
-				track_view->priv->hilighted_end = start + n_slots * track_view->priv->real_slot_size;
+				track_view->priv->hilighted_end = start + n_slots * track_view->priv->real_slot_width;
 				clutter_actor_queue_redraw(CLUTTER_ACTOR(track_view));
 			}
 		}
@@ -117,10 +172,10 @@ static void jammo_track_view_get_preferred_width(TangleActor* actor, gfloat for_
 	track_view = JAMMO_TRACK_VIEW(actor);
 	
 	if (min_width_p) {
-		*min_width_p = track_view->priv->n_slots * track_view->priv->slot_size;
+		*min_width_p = track_view->priv->n_slots * track_view->priv->slot_width;
 	}
 	if (natural_width_p) {
-		*natural_width_p = track_view->priv->n_slots * track_view->priv->slot_size;
+		*natural_width_p = track_view->priv->n_slots * track_view->priv->slot_width;
 	}
 	if (max_width_p) {
 		*max_width_p = 0.0;
@@ -128,71 +183,36 @@ static void jammo_track_view_get_preferred_width(TangleActor* actor, gfloat for_
 }
 
 static void jammo_track_view_get_preferred_height(TangleActor* actor, gfloat for_width, gboolean interacting, gfloat* min_height_p, gfloat* natural_height_p, gfloat* max_height_p) {
+	JammoTrackView* track_view;
+
+	track_view = JAMMO_TRACK_VIEW(actor);
+
 	if (min_height_p) {
-		*min_height_p = 80.0;
+		*min_height_p = track_view->priv->slot_height;
 	}
 	if (natural_height_p) {
-		*natural_height_p = 80.0;
+		*natural_height_p = track_view->priv->slot_height;
 	}
 	if (max_height_p) {
-		*max_height_p = 0.0;
+		*max_height_p = track_view->priv->slot_height;
 	}
 }
 
-static ClutterActor* do_grid(gfloat grid_size_x, gfloat grid_size_y, gfloat full_width, gint number_of_grids){
-	ClutterActor* grid = tangle_widget_new();
-	ClutterColor black = { 0,0,0, 255 };
-	ClutterColor grey = { 0,0,0, 50 };
-	int border_width_horizontal = 3;
-	int border_width_vertical = 1;
-
-	ClutterActor* top = clutter_rectangle_new_with_color (&black);
-	clutter_rectangle_set_border_color(CLUTTER_RECTANGLE(top),&black);
-	clutter_rectangle_set_border_width(CLUTTER_RECTANGLE(top),border_width_horizontal);
-	clutter_actor_set_size (top,full_width, border_width_horizontal);
-	clutter_actor_set_position (top,0,0);
-	clutter_container_add_actor (CLUTTER_CONTAINER (grid), top);
-
-	ClutterActor* bottom = clutter_rectangle_new_with_color (&black);
-	clutter_rectangle_set_border_color(CLUTTER_RECTANGLE(bottom),&black);
-	clutter_rectangle_set_border_width(CLUTTER_RECTANGLE(bottom),border_width_horizontal);
-	clutter_actor_set_size (bottom,full_width, border_width_horizontal);
-	clutter_actor_set_position (bottom,0,grid_size_y);
-	clutter_container_add_actor (CLUTTER_CONTAINER (grid), bottom);
-
-	int i;
-	for (i=0;i<number_of_grids;i++){
-		ClutterActor* bar = clutter_rectangle_new_with_color (&grey);
-		clutter_rectangle_set_border_color(CLUTTER_RECTANGLE(bar),&grey);
-		clutter_rectangle_set_border_width(CLUTTER_RECTANGLE(bar),border_width_vertical);
-		clutter_actor_set_size (bar, border_width_vertical,grid_size_y);
-		clutter_actor_set_position (bar,grid_size_x*i,0);
-		clutter_container_add_actor (CLUTTER_CONTAINER (grid), bar);
-	}
-
-	return grid;
-}
 static void jammo_track_view_allocate_wrapped(TangleWrapperActor* wrapper_actor, ClutterActor* wrapped, const ClutterActorBox* box, ClutterAllocationFlags flags) {
 	JammoTrackView* track_view;
-	ClutterActor* grid;
 
 	track_view = JAMMO_TRACK_VIEW(wrapper_actor);
 
 	TANGLE_WRAPPER_ACTOR_CLASS(jammo_track_view_parent_class)->allocate_wrapped(wrapper_actor, wrapped, box, flags);
 	
-	track_view->priv->real_slot_size = (box->x2 - box->x1) / track_view->priv->n_slots;
+	track_view->priv->real_slot_width = (box->x2 - box->x1) / track_view->priv->n_slots;
 
 	if (track_view->priv->line_every_nth_slot) {
-		grid = tangle_grid_new(track_view->priv->real_slot_size * track_view->priv->line_every_nth_slot - (1.0 / track_view->priv->n_slots), box->y2 - box->y1 - 1.0);
-/*
-		grid = do_grid(track_view->priv->real_slot_size * track_view->priv->line_every_nth_slot - (1.0 / track_view->priv->n_slots), //width of one grid
-						76, //height of the grid
-						track_view->priv->n_slots * track_view->priv->real_slot_size, //     full width of area
-						track_view->priv->n_slots / track_view->priv->line_every_nth_slot +1 ); //number of needed grids
-*/
-		tangle_widget_set_background_actor(TANGLE_WIDGET(wrapped), grid);
+		tangle_grid_set_grid_spacing_x(track_view->priv->grid, track_view->priv->real_slot_width * track_view->priv->line_every_nth_slot - (1.0 / track_view->priv->n_slots));
+		tangle_grid_set_grid_spacing_y(track_view->priv->grid, box->y2 - box->y1 - 1.0);
 	} else {
-		tangle_widget_set_background_actor(TANGLE_WIDGET(wrapped), NULL);
+		tangle_grid_set_grid_spacing_x(track_view->priv->grid, 0.0);
+		tangle_grid_set_grid_spacing_y(track_view->priv->grid, 0.0);
 	}
 }
 
@@ -215,23 +235,24 @@ static gboolean jammo_track_view_handle_dropping(TangleDroppableActor* droppable
 		sample_button = JAMMO_SAMPLE_BUTTON(dragging->draggable_actor);
 		copy_of_sample_button = jammo_sample_button_new_from_existing(sample_button);
 
-		n_slots = (guint)(clutter_actor_get_width(CLUTTER_ACTOR(dragging->draggable_actor)) / track_view->priv->real_slot_size + 0.5);
-		center_x -= (n_slots - 1) * track_view->priv->real_slot_size / 2;
-		slot = center_x / track_view->priv->real_slot_size;
+		n_slots = (guint)(clutter_actor_get_width(CLUTTER_ACTOR(dragging->draggable_actor)) / track_view->priv->real_slot_width + 0.5);
+		center_x -= (n_slots - 1) * track_view->priv->real_slot_width / 2;
+		slot = center_x / track_view->priv->real_slot_width;
 
 		jammo_editing_track_add_sample(track_view->priv->track, jammo_sample_button_get_sample(JAMMO_SAMPLE_BUTTON(copy_of_sample_button)), slot * track_view->priv->slot_duration);
 
 		/* tangle_draggable_actor_set_floating_mode_dragged(TANGLE_DRAGGABLE_ACTOR(copy_of_sample_button), TANGLE_FLOATING_MODE_COLLAPSE); */
 		clutter_actor_set_x(copy_of_sample_button, x - dragging->anchor_x);
 		clutter_actor_set_y(copy_of_sample_button, y - dragging->anchor_y);
+		clutter_actor_set_height(copy_of_sample_button, track_view->priv->slot_height);
 		tangle_widget_add(widget, copy_of_sample_button, NULL);
 
-		clutter_actor_animate(copy_of_sample_button, CLUTTER_EASE_OUT_CUBIC, 300, "x", slot * track_view->priv->real_slot_size, "y", 0.0, NULL);
+		clutter_actor_animate(copy_of_sample_button, CLUTTER_EASE_OUT_CUBIC, 300, "x", slot * track_view->priv->real_slot_width, "y", 0.0, NULL);
 
-		move_others_right_if_overlapping(track_view, copy_of_sample_button, slot);
+		gems_add_new_sample_to_track(12, slot); //TODO loop_id
 	}
 	
-	track_view->priv->hilighted_start = track_view->priv->hilighted_end = 0.0;
+	track_view->priv->hilighted_start = track_view->priv->hilighted_end = -1.0;
 
 	return FALSE;
 }
@@ -281,6 +302,9 @@ static GObject* jammo_track_view_constructor(GType type, guint n_construct_prope
 
 	g_signal_connect(widget, "actor-added", G_CALLBACK(on_actor_added), object);
 	g_signal_connect(widget, "actor-removed", G_CALLBACK(on_actor_removed), object);
+	
+	JAMMO_TRACK_VIEW(object)->priv->grid = TANGLE_GRID(tangle_grid_new(0.0, 0.0));
+	tangle_widget_set_background_actor(TANGLE_WIDGET(tangle_wrapper_actor_get_wrapped(TANGLE_WRAPPER_ACTOR(object))), CLUTTER_ACTOR(JAMMO_TRACK_VIEW(object)->priv->grid));
 
 	return object;
 }
@@ -301,8 +325,12 @@ static void jammo_track_view_set_property(GObject* object, guint prop_id, const
 		case PROP_SLOT_DURATION:
 			track_view->priv->slot_duration = g_value_get_uint64(value);
 			break;
-		case PROP_SLOT_SIZE:
-			track_view->priv->slot_size = g_value_get_float(value);
+		case PROP_SLOT_HEIGHT:
+			track_view->priv->slot_height = g_value_get_float(value);
+			clutter_actor_queue_relayout(CLUTTER_ACTOR(track_view));
+			break;
+		case PROP_SLOT_WIDTH:
+			track_view->priv->slot_width = g_value_get_float(value);
 			clutter_actor_queue_relayout(CLUTTER_ACTOR(track_view));
 			break;
 		case PROP_LINE_EVERY_NTH_SLOT:
@@ -315,6 +343,9 @@ static void jammo_track_view_set_property(GObject* object, guint prop_id, const
 		case PROP_DISABLED_SLOTS_END:
 			track_view->priv->disabled_slots_end = g_value_get_uint(value);
 			break;
+		case PROP_EDITING_ENABLED:
+			jammo_track_view_set_editing_enabled(track_view, g_value_get_boolean(value));
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
 			break;
@@ -336,8 +367,11 @@ static void jammo_track_view_get_property(GObject* object, guint prop_id, GValue
 		case PROP_SLOT_DURATION:
 			g_value_set_uint64(value, track_view->priv->slot_duration);
 			break;
-		case PROP_SLOT_SIZE:
-			g_value_set_float(value, (track_view->priv->real_slot_size ? track_view->priv->real_slot_size : track_view->priv->slot_size));
+		case PROP_SLOT_HEIGHT:
+			g_value_set_float(value, track_view->priv->slot_height);
+			break;
+		case PROP_SLOT_WIDTH:
+			g_value_set_float(value, (track_view->priv->real_slot_width ? track_view->priv->real_slot_width : track_view->priv->slot_width));
 			break;
 		case PROP_LINE_EVERY_NTH_SLOT:
 			g_value_set_uint(value, track_view->priv->line_every_nth_slot);
@@ -348,6 +382,9 @@ static void jammo_track_view_get_property(GObject* object, guint prop_id, GValue
 		case PROP_DISABLED_SLOTS_END:
 			g_value_set_uint(value, track_view->priv->disabled_slots_end);
 			break;
+		case PROP_EDITING_ENABLED:
+			g_value_set_boolean(value, track_view->priv->editing_enabled);
+			break;
 	        default:
 		        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
 		        break;
@@ -420,18 +457,29 @@ static void jammo_track_view_class_init(JammoTrackViewClass* track_view_class) {
 	                                0, G_MAXUINT64, 0,
 	                                G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
 	/**
-	 * JammoTrackView:slot-size:
+	 * JammoTrackView:slot-height:
+	 *
+	 * The height of the slot in the screen (pixels, Clutter units).
+	 */
+	g_object_class_install_property(gobject_class, PROP_SLOT_HEIGHT,
+	                                g_param_spec_float("slot-height",
+	                                "Slot height",
+	                                "The height of the slot in the screen (pixels, Clutter units)",
+	                                0, G_MAXFLOAT, 0,
+	                                G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
+	/**
+	 * JammoTrackView:slot-width:
 	 *
 	 * The width of the slot in the screen (pixels, Clutter units).
 	 */
-	g_object_class_install_property(gobject_class, PROP_SLOT_SIZE,
-	                                g_param_spec_float("slot-size",
-	                                "Slot size",
+	g_object_class_install_property(gobject_class, PROP_SLOT_WIDTH,
+	                                g_param_spec_float("slot-width",
+	                                "Slot width",
 	                                "The width of the slot in the screen (pixels, Clutter units)",
 	                                0, G_MAXFLOAT, 0,
 	                                G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
 	/**
-	 * JammoTrackView:slot-size:
+	 * JammoTrackView:line-every-nth-slot:
 	 *
 	 * The number of slots after a separator line is drawn (0 to disable).
 	 */
@@ -463,17 +511,31 @@ static void jammo_track_view_class_init(JammoTrackViewClass* track_view_class) {
 	                                "The number of slots that are disabled (in terms of dropping) from the end of the track",
 	                                0, G_MAXUINT, 0,
 	                                G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
+	/**
+	 * JammoTrackView:editing enabled:
+	 *
+	 * Whether the track can be edited or not.
+	 */
+	g_object_class_install_property(gobject_class, PROP_EDITING_ENABLED,
+	                                g_param_spec_boolean("editing-enabled",
+	                                "Editing enabled",
+	                                "Whether the track can be edited or not",
+	                                TRUE,
+	                                G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
 
 	g_type_class_add_private (gobject_class, sizeof (JammoTrackViewPrivate));
 }
 
 static void jammo_track_view_init(JammoTrackView* track_view) {
 	track_view->priv = G_TYPE_INSTANCE_GET_PRIVATE(track_view, JAMMO_TYPE_TRACK_VIEW, JammoTrackViewPrivate);
+	track_view->priv->editing_enabled = TRUE;
 
 	track_view->priv->hilight_color.red = 255;
 	track_view->priv->hilight_color.green = 255;
 	track_view->priv->hilight_color.blue = 255;
 	track_view->priv->hilight_color.alpha = 100;
+
+	track_view->priv->hilighted_start = track_view->priv->hilighted_end = -1.0;
 }
 
 static void on_actor_added(TangleWidget* widget, ClutterActor* actor, gpointer user_data) {
@@ -487,10 +549,6 @@ static void on_actor_added(TangleWidget* widget, ClutterActor* actor, gpointer u
 	sample_button = JAMMO_SAMPLE_BUTTON(actor);
 	sample = jammo_sample_button_get_sample(sample_button);
 	
-	//If user makes first dragging, not clicking, we do not want hear click-message anymore
-	jammo_mentor_speak_once(jammo_mentor_get_default(), "/opt/jammo/mentor_speech/mentor_composing_first_click_fi.ogg");
-	jammo_mentor_speak_once(jammo_mentor_get_default(), "/opt/jammo/mentor_speech/mentor_composing_first_drag_fi.ogg");
-
 	g_assert(jammo_sample_get_editing_track(sample));
 }
 
@@ -504,38 +562,28 @@ static void on_actor_removed(TangleWidget* widget, ClutterActor* actor, gpointer
 	sample_button = JAMMO_SAMPLE_BUTTON(actor);
 	
 	jammo_editing_track_remove_sample(track_view->priv->track, jammo_sample_button_get_sample(sample_button));
+
+	guint slot = clutter_actor_get_x(actor) / track_view->priv->slot_width;
+	gems_remove_sample_from_slot(slot);
 }
 
-static void on_animation_completed(ClutterAnimation* animation, gpointer user_data) {
-	TangleVault* vault;
-	ClutterContainer* container;
-	ClutterActor* actor;
-	
-	vault = TANGLE_VAULT(user_data);
-	tangle_vault_get(vault, 2, CLUTTER_TYPE_CONTAINER, &container, CLUTTER_TYPE_ACTOR, &actor);
-
-	clutter_container_remove_actor(container, actor);
-}
-
-static void move_others_right_if_overlapping(JammoTrackView* track_view, ClutterActor* sample_button, guint slot_index) {
+static gboolean check_if_overlapping(JammoTrackView* track_view, ClutterActor* sample_button, guint slot_index) {
 	TangleWidget* widget;
 	guint n_slots;
 	GList* children;
 	ClutterActor* actor = NULL;
 	gfloat x;
-	ClutterAnimation* animation;
-	TangleVault* vault;
 	
 	widget = TANGLE_WIDGET(tangle_wrapper_actor_get_wrapped(TANGLE_WRAPPER_ACTOR(track_view)));
-	n_slots = (guint)(clutter_actor_get_width(sample_button) / track_view->priv->real_slot_size + 0.5);
+	n_slots = (guint)(clutter_actor_get_width(sample_button) / track_view->priv->real_slot_width + 0.5);
 	for (children = tangle_widget_get_children_readonly(widget); children; children = children->next) {
 		actor = CLUTTER_ACTOR(children->data);
 		x = clutter_actor_get_x(actor);
 		if (actor != sample_button &&
-		    x < slot_index * track_view->priv->real_slot_size &&
-		    x + clutter_actor_get_width(actor) > slot_index * track_view->priv->real_slot_size) {
-			n_slots += slot_index - (guint)(x / track_view->priv->real_slot_size);
-			slot_index = (guint)(x / track_view->priv->real_slot_size);
+		    x < slot_index * track_view->priv->real_slot_width &&
+		    x + clutter_actor_get_width(actor) > slot_index * track_view->priv->real_slot_width) {
+		    n_slots += slot_index - (guint)(x / track_view->priv->real_slot_width);
+			slot_index = (guint)(x / track_view->priv->real_slot_width);
 			break;
 		}
 		actor = NULL;
@@ -545,29 +593,14 @@ static void move_others_right_if_overlapping(JammoTrackView* track_view, Clutter
 			actor = CLUTTER_ACTOR(children->data);
 			x = clutter_actor_get_x(actor);
 			if (actor != sample_button &&
-			    x >= slot_index * track_view->priv->real_slot_size &&
-			    x <= (slot_index + n_slots - 1) * track_view->priv->real_slot_size) {
-		    		n_slots -= (guint)(x / track_view->priv->real_slot_size) - slot_index;
+			    x >= slot_index * track_view->priv->real_slot_width &&
+			    x <= (slot_index + n_slots - 1) * track_view->priv->real_slot_width) {
+				n_slots -= (guint)(x / track_view->priv->real_slot_width) - slot_index;
 				break;
 			}
 			actor = NULL;
 		}
 	}
-	if (actor) {
-		for (children = tangle_widget_get_children_readonly(widget); children; children = children->next) {
-			actor = CLUTTER_ACTOR(children->data);
-			x = clutter_actor_get_x(actor);
-			if (actor != sample_button && x >= slot_index * track_view->priv->real_slot_size) {
-				x += n_slots * track_view->priv->real_slot_size;
-				animation = clutter_actor_animate(actor, CLUTTER_EASE_OUT_CUBIC, 500, "x", x, NULL);
-				if (x + clutter_actor_get_width(actor) > (track_view->priv->n_slots - track_view->priv->disabled_slots_end) * track_view->priv->real_slot_size) {
-					vault = tangle_vault_new(2, CLUTTER_TYPE_CONTAINER, widget, CLUTTER_TYPE_ACTOR, actor);
-					tangle_signal_connect_vault(animation, "completed", G_CALLBACK(on_animation_completed), vault);
-				} else {
-					jammo_editing_track_add_sample(track_view->priv->track, jammo_sample_button_get_sample(JAMMO_SAMPLE_BUTTON(actor)),
-					                               x / track_view->priv->real_slot_size * track_view->priv->slot_duration);
-				}
-			}
-		}
-	}
+	
+	return actor != NULL;
 }
