@@ -39,7 +39,7 @@ GST_ELEMENT_DETAILS ("JamMo Virtual Instrument",
     "Creates virtual instrument",
     "Aapo Rantalainen <aapo.rantalainen@gmail.com>, Mikko Gynther <mikko.gynther@lut.fi>");
 
-#define DEFAULT_SAMPLES_PER_BUFFER   1024
+#define DEFAULT_SAMPLES_PER_BUFFER   256
 #define DEFAULT_VOLUME               0.6
 #define DEFAULT_IS_LIVE              FALSE
 #define DEFAULT_TIMESTAMP_OFFSET     G_GINT64_CONSTANT (0)
@@ -53,15 +53,16 @@ enum
   PROP_CATEGORY,
   PROP_INITIALIZE_INSTRUMENT,
   PROP_INSTRUMENT_FOLDER,
-  PROP_NOTE_ON,
-  PROP_NOTE_OFF,
-  PROP_NOTE_TOGGLE,
+  PROP_ADD_EVENT,
+  PROP_REMOVE_EVENT,
+  PROP_CLEAR_EVENTS,
   PROP_VOLUME,
   PROP_IS_LIVE,
   PROP_TIMESTAMP_OFFSET,
   PROP_CAN_ACTIVATE_PUSH,
   PROP_CAN_ACTIVATE_PULL,
   PROP_RECORDING,
+  PROP_EVENTLIST,
   PROP_LAST
 };
 
@@ -126,18 +127,18 @@ gst_jammo_sampler_base_init (gpointer g_class)
   gst_element_class_set_details (element_class, &gst_jammo_sampler_details);
 }
 
-
-/*This is used when user asks name of note*/
-static gchar notenames[] ={'c','1','d','2','e','f',
-			   '3','g','4','a','5','b'};
+static gchar notenames[] = { 'c', '1', 'd', '2', 'e', 'f',
+  '3', 'g', '4', 'a', '5', 'b'
+};
 
 /*This is used when user pass char to instrument.
 It is best that user can use chars when calling instrument*/
 static gint get_note(gchar ch){
   int i;
-  for (i=0;i<NUMBER_OF_NOTES;i++) {
-    if (ch==notenames[i])
+  for (i=0;i<12;i++) {
+    if (ch==notenames[i]) {
       return i;
+		}
   }
 return -1; //error case
 }
@@ -172,18 +173,20 @@ static void gst_jammo_sampler_class_init (GstJammoSamplerClass * klass)
       g_param_spec_string ("instrument-folder", "Virtual Instruments basefolder",
           "instruments basefolder", "",
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, PROP_NOTE_ON,
-      g_param_spec_string ("note-on", "Add Note", "Given note is added",
-          NULL,
-	  G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_STATIC_STRINGS ));
-  g_object_class_install_property (gobject_class, PROP_NOTE_OFF,
-      g_param_spec_string ("note-off", "Remove Note", "Given note is removed",
-          NULL,
-          G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_STATIC_STRINGS ));
-  g_object_class_install_property (gobject_class, PROP_NOTE_TOGGLE,
-      g_param_spec_string ("note-toggle", "Toggle Note", "State of note is reversed",
-          NULL,
-          G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_STATIC_STRINGS ));
+  g_object_class_install_property(gobject_class, PROP_REMOVE_EVENT,
+                                 g_param_spec_pointer ("remove-event",
+                                 "Removes both start and stop events",
+                                 "",
+                                 G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
+  g_object_class_install_property(gobject_class, PROP_ADD_EVENT,
+                                 g_param_spec_pointer ("add-event",
+                                 "Add event",
+                                 "",
+                                 G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (gobject_class, PROP_CLEAR_EVENTS,
+      g_param_spec_boolean ("clear-events", "Clear events",
+          "Clear event list", FALSE,
+          G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_VOLUME,
       g_param_spec_double ("volume", "Volume", "Volume of test signal", 0.0,
           1.0, DEFAULT_VOLUME,
@@ -210,6 +213,9 @@ static void gst_jammo_sampler_class_init (GstJammoSamplerClass * klass)
       g_param_spec_boolean ("realtime", "Realtime",
           "Realtime", TRUE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (gobject_class, PROP_EVENTLIST,
+      g_param_spec_pointer ("eventlist", "Get a list of events",
+          "Get a list of events", G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   gstbasesrc_class->set_caps = GST_DEBUG_FUNCPTR (gst_jammo_sampler_setcaps);
   gstbasesrc_class->is_seekable =
       GST_DEBUG_FUNCPTR (gst_jammo_sampler_is_seekable);
@@ -259,7 +265,7 @@ src->wav_based_instrument[index]->sampletable =NULL;
 
 //printf("\nbefore looking for same sample\n");
 
-for (i=0;i<NUMBER_OF_OCTAVES*NUMBER_OF_NOTES;i++) {
+for (i=0;i<JAMMOMIDI_NUMBER_OF_NOTES;i++) {
 	if (src->wav_based_instrument[i]->filename!=NULL && strcmp(filename, src->wav_based_instrument[i]->filename)==0 && i!=index) {
 		src->wav_based_instrument[index]->sampletable=src->wav_based_instrument[i]->sampletable;
 		//printf("sampletable pointer ok\n");
@@ -307,10 +313,13 @@ static void gst_jammo_sampler_init (GstJammoSampler * src, GstJammoSamplerClass
   src->base_folder= "";
 
   //All notes start silently
-  for (i=0;i<NUMBER_OF_OCTAVES*NUMBER_OF_NOTES;i++)
+  for (i=0;i<JAMMOMIDI_NUMBER_OF_NOTES;i++)
     src->note_table[i]=FALSE;
 
   src->notes_playing=0;
+	src->last_event_processed=FALSE;
+
+	src->time=0;
 
   /* we operate in time */
   gst_base_src_set_format (GST_BASE_SRC (src), GST_FORMAT_TIME);
@@ -326,7 +335,7 @@ static void gst_jammo_sampler_init (GstJammoSampler * src, GstJammoSamplerClass
 //  gst_base_src_set_blocksize (GST_BASE_SRC (src), -1);
 
 
-  for (i=0;i<NUMBER_OF_OCTAVES*NUMBER_OF_NOTES;i++){
+  for (i=0;i<JAMMOMIDI_NUMBER_OF_NOTES;i++){
     src->wav_based_instrument[i] =malloc(sizeof(SlicedWav));
 		if (src->wav_based_instrument[i]==NULL) {
 		printf("error allocating memory. exiting\n");
@@ -685,47 +694,101 @@ static gfloat oneshot_sampler(GstJammoSampler * src, gint j) {
 
 
 static void do_playback_process(GstJammoSampler * src){
-	GstClockTime now = gst_clock_get_time(gst_system_clock_obtain()); //time just now		
-	GstClockTime current = GST_CLOCK_DIFF(src->playback_starting_time,now);
+	/*GstClockTime now = gst_clock_get_time(gst_system_clock_obtain()); //time just now		
+	GstClockTime current = GST_CLOCK_DIFF(src->playback_starting_time,now);*/
 
-	JammoMidiEvent* event = g_list_nth(src->eventlist,src->playback_index_for_event)->data;
-	GstClockTime timestamp = event->timestamp;
+	JammoMidiEvent* event;
+	GstClockTime timestamp;
 	/*
-	timestamp is for microseconds = µs
+	timestamp is in nanoseconds = ns
 	e.g.:
 	6.081 683 703 s
 	6 081.683 703 ms
-	6 081 683.703 ns
-	6 081 683 703 µs
+	6 081 683.703 µs
+	6 081 683 703 ns
 	*/
-	if (current > timestamp){
-		printf("DEBUG: gstjammosampler: delayed: %lu ms\n",(current - timestamp)/1000/1000); 
-		src->notes_playing++;
 
-		int octave=event->octave;
+	event = g_list_nth_data(src->eventlist,src->playback_index_for_event);
+	if (event==NULL)
+		{
+			int i=0;
+			gboolean finished=TRUE;
+			// check if all notes have finished playing
+			for(i=0;i<JAMMOMIDI_NUMBER_OF_NOTES;i++)
+				if (src->note_table[i]==TRUE)
+					finished=FALSE;
+
+			if (finished) {
+				src->eos_reached=TRUE;
+				src->playback_not_started=TRUE;
+				src->last_event_processed=FALSE;
+			}
+			return;
+		}
+	timestamp = event->timestamp;
+
+	while (src->time > timestamp){
+		printf("DEBUG: gstjammosampler: delayed: %llu ms\n",(unsigned long long) (src->time - timestamp)/1000/1000);
 		int note=event->note;
 
-		if (src->wav_based_instrument[octave*NUMBER_OF_NOTES+note]->filename!=NULL) {
-			src->note_table[octave*NUMBER_OF_NOTES+note]=TRUE;
-			src->wav_based_instrument[octave*NUMBER_OF_NOTES+note]->sampleindex=0;
-			src->wav_based_instrument[octave*NUMBER_OF_NOTES+note]->xfadecounter=-1.0;
-			src->wav_based_instrument[octave*NUMBER_OF_NOTES+note]->releasecounter=-1.0;
-			src->wav_based_instrument[octave*NUMBER_OF_NOTES+note]->released=FALSE;
+		if (event->type==0 && note>-1 && note < JAMMOMIDI_NUMBER_OF_NOTES) { //start  NOTE_ON
+			// if the note was not playing increment the number of playing notes
+			if (src->note_table[note]==FALSE)
+				src->notes_playing++;
+			// start playing the note from the beginning
+			if (src->wav_based_instrument[note]->filename!=NULL) {
+				src->note_table[note]=TRUE;
+				src->wav_based_instrument[note]->sampleindex=0;
+				src->wav_based_instrument[note]->xfadecounter=-1.0;
+				src->wav_based_instrument[note]->releasecounter=-1.0;
+				src->wav_based_instrument[note]->released=FALSE;
+			}
+			else {
+				// error with the sample file
+				src->note_table[note]=FALSE;
+				printf("gstjammosampler: error. Trying play note/octave not in sample file\n");
+			}
 		}
 
+		else if (event->type==1&& note>-1 && note < JAMMOMIDI_NUMBER_OF_NOTES) { //stop  NOTE_OFF
+			// release note only, the number of notes will be decreased when the note has faded
+			src->wav_based_instrument[note]->released=TRUE;
+		}
+		else {
+			printf("a faulty event in list\n");
+			printf("note: %d\n", note);
+		}
 
+		// move to the next event
 		src->playback_index_for_event++;
-		if (src->playback_index_for_event==g_list_length(src->eventlist))
-		{
-		src->eos_reached=TRUE; //FIXME: This stops it too early, that last note doesn't hear.
-		src->playback_not_started=TRUE;
+
+		// check if there are no more events
+		if (src->playback_index_for_event==g_list_length(src->eventlist)) {
+			int i=0;
+			src->last_event_processed=TRUE;
+			// release all notes just in case some note-off is missing
+			for(i=0;i<JAMMOMIDI_NUMBER_OF_NOTES;i++)
+				src->wav_based_instrument[i]->released=TRUE;
 		}
 
+		// get next event from the list
+		event = g_list_nth_data(src->eventlist,src->playback_index_for_event);
+		if (event==NULL)
+		{
+			return;
+		}
+		// get the time of the event
+		timestamp = event->timestamp;
 	}
+
 }
 
 #define DEFINE_PROCESS(type,scale) \
 static void gst_jammo_sampler_process_##type (GstJammoSampler * src, g##type * samples) {\
+	/* using a static number of maximum simultaneous notes to make sampler slightly lighter */\
+	/* malloc consumes some cycles and usually there is no need to play many notes simultaneously */ \
+	/* edit MAXNOTES in header file to change the number of maximum simultaneous notes */ \
+  static g##type values[MAXNOTES];\
 	if (!(src->recording)) /*we are in playback mode*/ \
 		do_playback_process(src); \
 \
@@ -746,11 +809,9 @@ static void gst_jammo_sampler_process_##type (GstJammoSampler * src, g##type * s
   gint i,j;\
   g##type value=0;\
 \
-  /*g##type values[src->notes_playing];*/\
-  g##type * values=malloc(src->notes_playing*sizeof(g##type));\
   gdouble amp; \
   gint counter=0;\
-  for (i=0;i<src->notes_playing;i++)\
+  for (i=0;i<MAXNOTES;i++)\
      values[i]=0.0;\
 \
   amp = used_volume * scale;\
@@ -761,7 +822,7 @@ static void gst_jammo_sampler_process_##type (GstJammoSampler * src, g##type * s
 \
       /* Find what tone must be processed */ \
       counter=0;\
-      for (j=0;j<NUMBER_OF_OCTAVES*NUMBER_OF_NOTES;j++) \
+      for (j=0;j<JAMMOMIDI_NUMBER_OF_NOTES;j++) \
 	  {\
 	  if (src->note_table[j]) \
 	    {\
@@ -772,6 +833,8 @@ static void gst_jammo_sampler_process_##type (GstJammoSampler * src, g##type * s
 	    if (src->category==GST_JAMMO_SAMPLER_SAMPLER_ONESHOT)\
 		values[counter]=(g##type)(amp*oneshot_sampler(src, j));\
 	    counter++;\
+			if (counter>=MAXNOTES-1) \
+				break; \
 	    /* end if note is playing */ \
 	    } \
 	  /* end processing tones */ \
@@ -783,7 +846,7 @@ static void gst_jammo_sampler_process_##type (GstJammoSampler * src, g##type * s
       else \
 	  { \
 	  value=values[0];	\
-	  for (j=1;j<src->notes_playing;j++)\
+	  for (j=1;j<counter;j++)\
 	  /* Hard clipping. When adding values we may end up */ \
 	  /*with too big values. Although very nasty this sounds much */ \
 	  /*better than data type overflows */ \
@@ -796,7 +859,6 @@ static void gst_jammo_sampler_process_##type (GstJammoSampler * src, g##type * s
 	  } \
 \
       } \
-   free(values);\
 }
 
 DEFINE_PROCESS (int16, 32767.0);
@@ -816,9 +878,15 @@ static JammoSamplerProcessFunc process_funcs[] = {
 static void gst_jammo_sampler_get_times (GstBaseSrc * basesrc, GstBuffer * buffer,
     GstClockTime * start, GstClockTime * end)
 {
+	GstJammoSampler *src = GST_JAMMO_SAMPLER (basesrc);
+
+	GstClockTime timestamp = GST_BUFFER_TIMESTAMP (buffer);
+	// store time of buffer, it is used in playback mode for processing events
+	// and in recording mode for timestamping
+	src->time = timestamp;
+
   /* for live sources, sync on the timestamp of the buffer */
   if (gst_base_src_is_live (basesrc)) {
-    GstClockTime timestamp = GST_BUFFER_TIMESTAMP (buffer);
 
     if (GST_CLOCK_TIME_IS_VALID (timestamp)) {
       /* get duration to calculate end time */
@@ -848,7 +916,7 @@ static gboolean gst_jammo_sampler_start (GstBaseSrc * basesrc)
   src->check_seek_stop = FALSE;
   src->eos_reached = FALSE;
   src->tags_pushed = FALSE;
-  for (i=0; i<NUMBER_OF_OCTAVES*NUMBER_OF_NOTES;i++)
+  for (i=0; i<JAMMOMIDI_NUMBER_OF_NOTES;i++)
        src->accumulators[i] = 0;
 
 
@@ -863,7 +931,7 @@ static gboolean gst_jammo_sampler_start (GstBaseSrc * basesrc)
   else  {
     printf("DEBUG:gstjammosampler: not recording, playback\n");
 
-    src->playback_starting_time = gst_clock_get_time(gst_system_clock_obtain());
+    //src->playback_starting_time = gst_clock_get_time(gst_system_clock_obtain());
     src->playback_not_started=FALSE;
     src->playback_index_for_event=0;
 
@@ -873,7 +941,7 @@ static gboolean gst_jammo_sampler_start (GstBaseSrc * basesrc)
 
    for (list = src->eventlist; list; list = list->next) {
       n = (list->data);
-      printf("TEST:gstjammosampler:In stack: %lu µs: note:%d octave: %d toggled\n", n->timestamp,n->note,n->octave); 
+      printf("TEST:gstjammosampler:In stack: %llu ms: note:%d ,type: %s \n", (unsigned long long)n->timestamp/1000/1000,n->note,n->type==0? "start":"stop");
    }
   }
 
@@ -966,10 +1034,14 @@ static GstFlowReturn gst_jammo_sampler_create (GstBaseSrc * basesrc, guint64 off
 
   /* if no length was given, use our default length in samples otherwise convert
    * the length in bytes to samples. */
-  if (length == -1)
+  /*if (length == -1)
     samples = src->samples_per_buffer;
   else
-    samples = length / src->sample_size;
+    samples = length / src->sample_size;*/
+
+	// we need to use smaller buffer than requested
+	// otherwise there will be too much latency
+	samples = src->samples_per_buffer;
 
   /* if no offset was given, use our next logical byte */
   if (offset == -1)
@@ -1206,12 +1278,12 @@ gst_jammo_sampler_set_property (GObject * object, guint prop_id,
 		//printf("base_folder='%s', token='%s', filename='%s'\n",src->base_folder,token,filename);
 
 		get_token(&token);
-		base= get_note(token[0])+NUMBER_OF_NOTES*atoi(&token[1]);
+		base= get_note(token[0])+12*atoi(&token[1]);
 		get_token(&token);
-		min = get_note(token[0])+NUMBER_OF_NOTES*atoi(&token[1]);
+		min = get_note(token[0])+12*atoi(&token[1]);
 		get_token(&token);
-		max = get_note(token[0])+NUMBER_OF_NOTES*atoi(&token[1]);
-		//printf("base:%d,min:%d, max:%d\n",base,min,max);
+		max = get_note(token[0])+12*atoi(&token[1]);
+		//printf("base: %d,min:%d, max:%d\n", base,min,max);
 		
 		get_token(&token);
 		loop_start=atoi(token);
@@ -1246,74 +1318,140 @@ gst_jammo_sampler_set_property (GObject * object, guint prop_id,
 
       break;
 
-    case PROP_NOTE_ON:
-    case PROP_NOTE_OFF:
-    case PROP_NOTE_TOGGLE:
-	;
 
-	const gchar *str = g_value_get_string(value);
-	gchar oc=str[1];
-	gchar no=str[0];
+		case PROP_REMOVE_EVENT:
+			;
+			JammoMidiEvent* event = (JammoMidiEvent*)(g_value_get_pointer(value));
+			unsigned char note_ = event->note;
 
-	int octave=oc-48;
-	int note=get_note(no);
+			printf("remove event\n");
+			GstClockTime time= event->timestamp;
 
-	if (note==-1)
-	    break;
-	if (octave<0 || octave>=NUMBER_OF_OCTAVES)
-	    break;
 
-	gboolean old_value=src->note_table[octave*NUMBER_OF_NOTES+note];
+			JammoMidiEvent* candidate_for_removal=NULL;
+			GList* list;
+			JammoMidiEvent* n;
 
-	//if PROP_NOTE_OFF
-	/* If note-off received, released variable in sliced wav will be set to TRUE
-	Note will be set to off state when it has been released properly but before
-	that it is playing */
+			//printf("Removing this: %llu ms: note:%d \n",(unsigned long long)time,note_);
+			for (list = src->eventlist; list; list = list->next) {
+				//printf(".\n");
+				n = (list->data);
+				//printf("Scanning stack: %llu ms: note:%d ,type: %s \n", (unsigned long long)n->timestamp,get_note(n->note), n->type==JAMMOMIDI_NOTE_ON? "start":"stop");
 
-	gboolean new_value = FALSE;
-	if (prop_id==PROP_NOTE_ON)
-	  new_value=TRUE;
+				//First find last START
+				if (note_!= n->note)
+					{
+					//printf(" This is not asked note-octave \n");
+					continue;
+					}
+				//printf(" This is asked note-octave\n");
 
-	if (prop_id==PROP_NOTE_TOGGLE)
-	  new_value=!old_value;
+				if (n->type==JAMMOMIDI_NOTE_OFF && candidate_for_removal==NULL) {
+					//printf("  We are not yet found any start, and this is stop, so keep looking.\n");
+					continue;
+				}
 
-	if (new_value==TRUE) {
+				if (time >= n->timestamp )
+					{
+						//printf("time is correct\n");
+						if (n->type==JAMMOMIDI_NOTE_ON)
+							{
+							//printf("   START found\n");
+							candidate_for_removal=n;
+							//printf("   set this for candidate_for_removal\n");
+							}
+					}
+
+					else{
+							if (n->type==JAMMOMIDI_NOTE_OFF) {
+								//printf("STOP found\n");
+								//printf("Now DELETE candidate (i.e start) and this stop\n");
+								src->eventlist = g_list_remove(src->eventlist,n);
+								g_free(n);
+							}
+
+							else if (n->type==JAMMOMIDI_NOTE_ON) {
+								//printf("Second start without stop, but this is not asked (because of time)\n");
+								//printf("Delete only candidate (i.e previous start)\n");
+							}
+
+							src->eventlist = g_list_remove(src->eventlist,candidate_for_removal);
+							g_free(candidate_for_removal);
+							candidate_for_removal=NULL;
+							continue;
+						}
+
+			}
+
+			//If candidate_for_removal is set but not removed, remove it now
+			if (candidate_for_removal !=NULL) {
+				//printf("We found only start event, but not stop. Remove that start now.\n");
+				src->eventlist = g_list_remove(src->eventlist,candidate_for_removal);
+				g_free(candidate_for_removal);
+			}
+
+		//printf("removing event done\n");
+			break;
+
+    case PROP_ADD_EVENT:
+			;
+			//JammoMidiEvent* event;
+			event = (JammoMidiEvent*)(g_value_get_pointer(value));
+			unsigned char note = event->note;
+
+			//realtime. There are no timestamp in event, use current time
+			if (src->recording){
+			printf("realtime\n");
+			GstClockTime timestamp;
+			timestamp = src->time;
+			event->timestamp=timestamp;
+
+		if (event->type==JAMMOMIDI_NOTE_ON) {
 		/* It should be checked that note can be played, i.e. not all sampler
 		instruments can play c0. Playing notes that are not possible to play was a
 		likely source of segmentation faults with jammo's VI mode. no segfaults after
 		this code was added */
-		if (src->wav_based_instrument[octave*NUMBER_OF_NOTES+note]->filename!=NULL) {
-			src->note_table[octave*NUMBER_OF_NOTES+note]=TRUE;
-			src->wav_based_instrument[octave*NUMBER_OF_NOTES+note]->sampleindex=0;
-			src->wav_based_instrument[octave*NUMBER_OF_NOTES+note]->xfadecounter=-1.0;
-			src->wav_based_instrument[octave*NUMBER_OF_NOTES+note]->releasecounter=-1.0;
-			src->wav_based_instrument[octave*NUMBER_OF_NOTES+note]->released=FALSE;
+		if (note>-1 && note < JAMMOMIDI_NUMBER_OF_NOTES) {
+			if (src->wav_based_instrument[note]->filename!=NULL) {
+				if (src->note_table[note]==FALSE)
+					src->notes_playing++;
+				src->note_table[note]=TRUE;
+				src->wav_based_instrument[note]->sampleindex=0;
+				src->wav_based_instrument[note]->xfadecounter=-1.0;
+				src->wav_based_instrument[note]->releasecounter=-1.0;
+				src->wav_based_instrument[note]->released=FALSE;
+			}
+			else {
+				// error with the sample file
+				src->note_table[note]=FALSE;
+				printf("gstjammosampler: error. Trying add note/octave not in sample file\n");
+			}
 		}
 	}
-	else 
-		src->wav_based_instrument[octave*NUMBER_OF_NOTES+note]->released=TRUE;
+	else if (note>-1 && note < JAMMOMIDI_NUMBER_OF_NOTES) { //JAMMOMIDI_NOTE_OFF
+		// release note only, the number of notes will be decreased when the note has faded
+		src->wav_based_instrument[note]->released=TRUE;
+	}
 
-	//We must keep track how many notes are playing (we do not want count them separatedly)
-	if (new_value && !old_value) /*one FALSE is now TRUE*/
-	    src->notes_playing++;
-	
-	//Add this event to list.
-	JammoMidiEvent* event;
 
-	event = malloc(sizeof(JammoMidiEvent));
-	event->note=note;
-	event->octave=octave;
+	}
 
-	GstClockTime now = gst_clock_get_time(gst_system_clock_obtain()); //time just now	
+	else { //adding non-realtime (nothing special)
+			printf("non-realtime\n");
+			}
 
-	event->timestamp=GST_CLOCK_DIFF(src->creation_time,now);
-	printf("DEBUG: gstjammosampler: added event %lu ms: note:%d octave:%d toggled\n",event->timestamp/1000/1000,event->note,event->octave); 
-	
-	src->eventlist = g_list_append(src->eventlist,event);
+	src->eventlist = g_list_insert_sorted(src->eventlist ,event,compare_event_time);
+
+
+	printf("DEBUG: gstjammosampler: added event %llu ms: note:'%u'(%s %d) ,type=%s\n",(unsigned long long)(event->timestamp/1000/1000),note,jammomidi_note_to_char(note),note/12,event->type==JAMMOMIDI_NOTE_ON? "start":"stop");
 
-	
 	break;
-
+    case PROP_CLEAR_EVENTS:
+      if (g_value_get_boolean (value)) {
+	      jammomidi_free_glist(&(src->eventlist));
+	      src->eventlist=NULL;
+      }
+      break;
     case PROP_VOLUME:
       src->volume = g_value_get_double (value);
       break;
@@ -1332,6 +1470,16 @@ gst_jammo_sampler_set_property (GObject * object, guint prop_id,
     case PROP_RECORDING:
       src->recording = g_value_get_boolean (value);
       break;
+   case PROP_EVENTLIST:
+      //We got GList containing JammoMidiEvents, take all one-by-one and add to own internal eventlist
+      for (list = g_value_get_pointer (value); list; list = list->next) {
+        n = (list->data);
+        //printf("CASE PROP_EVENTLIST:gstjammosampler:Add: %llu ns: note:%d ,type: %s \n", (unsigned long long)n->timestamp,n->note, n->type==0? "start":"stop");
+				// we want to allocate memory for the event so that nothing else will delete events
+				// from samplers list
+				jammomidi_store_event_to_glist(&(src->eventlist), n->note, n->type, n->timestamp);
+        }
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1359,20 +1507,7 @@ static void gst_jammo_sampler_get_property (GObject * object, guint prop_id,
     case PROP_INSTRUMENT_FOLDER:
       g_value_set_string (value, src->base_folder);
       break;
-    case PROP_NOTE_ON:
-    case PROP_NOTE_OFF:
-    case PROP_NOTE_TOGGLE:
-      ;
-      gchar *dumb;
-      int i,j;
-      dumb = g_strdup_printf("Notes:\n");
-      for (i=0;i<NUMBER_OF_OCTAVES;i++)
-	  {
-	  for (j=0;j<NUMBER_OF_NOTES;j++)
-	      dumb = g_strdup_printf("%s%d%c=%s ",dumb,i,notenames[j],src->note_table[i*NUMBER_OF_NOTES+j]? "On ":"Off");
-	  dumb = g_strdup_printf("%s\n",dumb);
-	  }
-      g_value_set_string (value, dumb);
+    case PROP_ADD_EVENT:
       break;
     case PROP_VOLUME:
       g_value_set_double (value, src->volume);
@@ -1392,6 +1527,9 @@ static void gst_jammo_sampler_get_property (GObject * object, guint prop_id,
     case PROP_RECORDING:
       g_value_set_boolean (value, src->recording);
       break;
+		case PROP_EVENTLIST:
+			g_value_set_pointer (value, src->eventlist);
+			break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
