@@ -1,11 +1,17 @@
 /*
- * test-stample-button.c
+ * test-sequencer
  *
  * This file is part of JamMo.
  *
+ * Most left image is sample-button (can be dragged to track)
+ * Button on top, removes sample from slot 5. (there are two slots in same grid)
+ * Bottom left adds new sample to slot 1.
+ * Bottom right is play button.
+ *
  * (c) 2009 University of Oulu
  *
  * Authors: Henrik Hedberg <henrik.hedberg@oulu.fi>
+ *          Aapo Rantalainen
  */
 
 /**********
@@ -30,6 +36,13 @@ static gboolean create_new_sample(TangleButton* button, gpointer track_view) {
 	return FALSE;
 }
 
+
+static gboolean remove_sample(TangleButton* button, gpointer track_view) {
+	jammo_track_view_remove_jammo_sample_button_from_slot(JAMMO_TRACK_VIEW(track_view),  5); //slot_number
+	return FALSE;
+}
+
+
 static gboolean on_play_button_clicked(TangleButton* button, gpointer user_data) {
 	jammo_sequencer_play(JAMMO_SEQUENCER(user_data));
 
@@ -50,6 +63,7 @@ int main(int argc, char** argv) {
 	ClutterActor* cursor;
 	ClutterActor* play_button;
 	ClutterActor* button_for_creating;
+	ClutterActor* button_for_removing;
 	
 	jammo_meam_init(&argc, &argv);
 	jammo_chum_init(&argc, &argv);
@@ -80,7 +94,7 @@ int main(int argc, char** argv) {
 	track = jammo_editing_track_new();
 	jammo_sequencer_add_track(sequencer, JAMMO_TRACK(track));
 	
-	track_view = jammo_track_view_new(track, 25, 2181818182LL, 50.0);
+	track_view = jammo_track_view_new(track, 25, 2181818182LL, 50.0, 80.0);
 	g_object_set(track_view, "line-every-nth-slot", 2, 
 	             "disabled-slots-begin", 1,
 		     "disabled-slots-end", 1, NULL);
@@ -113,6 +127,12 @@ int main(int argc, char** argv) {
 	g_signal_connect(button_for_creating, "clicked", G_CALLBACK(create_new_sample), track_view);
 	clutter_container_add(CLUTTER_CONTAINER(box), button_for_creating, NULL);
 	
+	button_for_removing = tangle_button_new();
+	clutter_actor_set_position(button_for_removing, 200.0, 0.0);
+	tangle_widget_add_after(TANGLE_WIDGET(button_for_removing), clutter_texture_new_from_file("test_image_100x74.jpg", NULL), NULL, NULL, NULL);
+	g_signal_connect(button_for_removing, "clicked", G_CALLBACK(remove_sample), track_view);
+	clutter_container_add(CLUTTER_CONTAINER(box), button_for_removing, NULL);
+
 	clutter_actor_show_all(stage);
 	clutter_main();
 	
